package no.ntnu.idatt1002.demo.data.recipes;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class RecipeRegisterTest {

    RecipeRegister myRecipes;

    @BeforeEach
    void beforeEach() {
        myRecipes = new RecipeRegister();
        myRecipes.addRecipe(new Recipe("Pasta al Limone", "Instructions"));
    }

    @Test
    @DisplayName("Adding recipe to register successfully.")
    void addRecipeToRegister() {
        int numberOfRecipes = myRecipes.getRecipes().size();
        myRecipes.addRecipe(new Recipe("Carbonara", "Instructions"));
        assertEquals(numberOfRecipes + 1, myRecipes.getRecipes().size());
    }

    @Test
    @DisplayName("Throw exception when adding already registered recipe.")
    void recipeAlreadyInRegister() {
        assertThrows(IllegalArgumentException.class, () -> myRecipes.addRecipe( new Recipe("Pasta al Limone", "Instructions")));
    }

    @Test
    @DisplayName("Throw exception when adding null recipe.")
    void addNullRecipe() {
        assertThrows(NullPointerException.class, () -> myRecipes.addRecipe(null));
    }

    @Test
    @DisplayName("Returns correct recipe based on name.")
    void getRecipeByName() {
        assertEquals(new Recipe("Pasta al Limone", "Instructions"), myRecipes.getRecipe("Pasta al Limone"));
    }

}