package no.ntnu.idatt1002.demo.data.recipes;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class RecipeTest {

    Recipe recipe;

    @BeforeEach
    void beforeEach() {
        recipe = new Recipe("Meat, cheese and potatoes", "Instructions");

        recipe.addIngredient(FoodItem.MINCED_MEAT, 500, MeasuringUnit.GR);
        recipe.addIngredient(FoodItem.POTATO, 750, MeasuringUnit.GR);
        recipe.addIngredient(FoodItem.YELLOW_CHEESE, 2, MeasuringUnit.DL);

    }

    @Test
    @DisplayName("Constructor creates Recipe successfully.")
    void constructorCreatesRecipe() {
        assertEquals(recipe, new Recipe("Meat, cheese and potatoes", "Instructions"));
        // Recipes are equal if name and Instructions are equal.
    }

    @Test
    @DisplayName("Constructor throws exception for empty name or instruction.")
    void constructorThrowsException() {
        assertThrows(IllegalArgumentException.class, () -> new Recipe("", "Instructions"));
        assertThrows(IllegalArgumentException.class, () -> new Recipe("Name", ""));

    }

    @Test
    @DisplayName("Can change name of recipe successfully.")
    void setNameSuccessfully() {
        String newName = "Meat and cheese";
        recipe.setName(newName);
        assertEquals(newName, recipe.getName());
    }

    @Test
    @DisplayName("Throws exception when name set to empty string.")
    void setNameEmptyThrowsException() {
        assertThrows(IllegalArgumentException.class, () -> recipe.setName(" "));
    }

    @Test
    @DisplayName("Can change instructions of recipe successfully.")
    void setInstructionsSuccessfully() {
        String newInstruction = "New instructions";
        recipe.setInstructions(newInstruction);
        assertEquals(newInstruction, recipe.getInstructions());
    }

    @Test
    @DisplayName("Throws exception when Instructions set to empty string.")
    void setInstructionsEmptyThrowsException() {
        assertThrows(IllegalArgumentException.class, () -> recipe.setInstructions(""));
    }


    @Test
    @DisplayName("Ingredient is added to recipe successfully.")
    void addNewIngredient() {
        int ingredientsInRecipe = recipe.getIngredientList().size();
        assertTrue(recipe.addIngredient(FoodItem.LEMON, 1, MeasuringUnit.PC));
        assertEquals(ingredientsInRecipe + 1, recipe.getIngredientList().size());
    }

    @Test
    @DisplayName("Ingredient already in recipe does not alter and returns true.")
    void addAlreadyIncludedIngredient() {
        int ingredientsInRecipe = recipe.getIngredientList().size();
        assertTrue(recipe.addIngredient(FoodItem.MINCED_MEAT, 500, MeasuringUnit.GR));
        assertEquals(ingredientsInRecipe, recipe.getIngredientList().size());
    }

    @Test
    @DisplayName("Invalid ingredient change is not made and false is returned.")
    void addInvalidIngredientReturnsFalse() {
        int ingredientsInRecipe = recipe.getIngredientList().size();
        assertFalse(recipe.addIngredient(null, 500, MeasuringUnit.GR));
        assertEquals(ingredientsInRecipe, recipe.getIngredientList().size());
    }


    @Test
    @DisplayName("Updating ingredient status works as expected.")
    void updateIngredientStatus() {
        IngredientsAtHand inFridge = new IngredientsAtHand();
        inFridge.addIngredient(FoodItem.MINCED_MEAT);
        inFridge.addIngredient(FoodItem.YELLOW_CHEESE);


        recipe.getIngredientList().forEach((ingredient) -> assertFalse(ingredient.isAtHand()));
        recipe.updateIngredientStatus(inFridge);

        assertTrue(recipe.getIngredient(FoodItem.MINCED_MEAT).isAtHand());
        assertTrue(recipe.getIngredient(FoodItem.YELLOW_CHEESE).isAtHand());
        assertFalse(recipe.getIngredient(FoodItem.POTATO).isAtHand());
    }

    @Test
    @DisplayName("Update of ingredient status based on empty or null ingredients at hand collection.")
    void nullOrEmptyIngredientsAtHand() {
        IngredientsAtHand emptyFridge = new IngredientsAtHand();

        assertThrows(NullPointerException.class, () -> recipe.updateIngredientStatus(null));
        assertThrows(IllegalArgumentException.class, () -> recipe.updateIngredientStatus(emptyFridge));
    }

}